@@ -1,20 +1,26 @@
 #
 # close3Dpairs.R
 #
-#   $Revision: 1.6 $   $Date: 2015/10/20 04:03:31 $
+#   $Revision: 1.8 $   $Date: 2016/03/06 11:18:54 $
 #
 #  extract the r-close pairs from a 3D dataset
 # 
 #
 closepairs.pp3 <- local({
 
-  closepairs.pp3 <- function(X, rmax, ordered=TRUE,
-                           what=c("all", "indices"), ...) {
+  closepairs.pp3 <- function(X, rmax, twice=TRUE,
+                             what=c("all", "indices"),
+                             distinct=TRUE, neat=TRUE, ...) {
     verifyclass(X, "pp3")
     what <- match.arg(what)
     stopifnot(is.numeric(rmax) && length(rmax) == 1)
     stopifnot(is.finite(rmax))
     stopifnot(rmax >= 0)
+    ordered <- list(...)$ordered
+    if(missing(twice) && !is.null(ordered)) {
+      warning("Obsolete argument 'ordered' has been replaced by 'twice'")
+      twice <- ordered
+    }
     npts <- npoints(X)
     nama <- switch(what,
                    all = c("i", "j",
@@ -34,6 +40,12 @@ closepairs.pp3 <- local({
     ## First make an OVERESTIMATE of the number of pairs
     nsize <- ceiling(5 * pi * (npts^2) * (rmax^3)/volume(as.box3(X)))
     nsize <- max(1024, nsize)
+    if(nsize > .Machine$integer.max) {
+      warning(
+        "Estimated number of close pairs exceeds maximum possible integer",
+        call.=FALSE)
+      nsize <- .Machine$integer.max
+    }
     ## Now extract pairs
     XsortC <- coords(Xsort)
     x <- XsortC$x
@@ -60,14 +72,56 @@ closepairs.pp3 <- local({
     ## convert i,j indices to original sequence
     a$i <- oo[a$i]
     a$j <- oo[a$j]
-    ## are (i, j) and (j, i) equivalent?
-    if(!ordered) {
+    ## handle options
+    if(twice) {
+      ## both (i, j) and (j, i) should be returned
       a <- as.data.frame(a)
-      a <- a[with(a, i < j), , drop=FALSE]
-      a <- as.list(a)
+      a <- as.list(rbind(a, swapdata(a, what)))
+    } else if(neat) {
+      ## enforce i < j
+      swap <- with(a, (j < i))
+      if(any(swap)) {
+        a <- as.data.frame(a)
+        a[swap,] <- swapdata(a[swap, ,drop=FALSE], what)
+        a <- as.list(a)
+      }
     }
+    ## add pairs of identical points?
+    if(!distinct) {
+      ii <- seq_len(npts)
+      xtra <- data.frame(i = ii, j=ii)
+      if(what == "all") {
+        coo <- coords(X)[, c("x","y","z")]
+        zeroes <- rep(0, npts)
+        xtra <- cbind(xtra, coo, coo, zeroes, zeroes, zeroes, zeroes)
+      }
+      a <- as.list(rbind(as.data.frame(a), xtra))
+    }
+    ## done
     return(a)
   }
+
+  swapdata <- function(a, what) {
+    switch(what,
+           all = {
+             with(a, data.frame(i  =  j,
+                                j  =  i,
+                                xi =  xj,
+                                yi =  yj,
+                                zi =  zj,
+                                xj =  xi,
+                                yj =  yi,
+                                zj =  zi,
+                                dx = -dx,
+                                dy = -dy,
+                                dz = -dz,
+                                d  =  d))
+           },
+           indices = {
+             with(a, data.frame(i=j,
+                                j=i))
+           })
+  }
   
   nuttink <- function(x) numeric(0)
 
@@ -105,6 +159,12 @@ crosspairs.pp3 <- local({
     ## First (over)estimate the number of pairs
     nsize <- ceiling(3 * pi * (rmax^3) * nX * nY/volume(as.box3(Y)))
     nsize <- max(1024, nsize)
+    if(nsize > .Machine$integer.max) {
+      warning(
+        "Estimated number of close pairs exceeds maximum possible integer",
+        call.=FALSE)
+      nsize <- .Machine$integer.max
+    }
     ## .Call
     XsortC <- coords(Xsort)
     YsortC <- coords(Ysort)
