@@ -3,13 +3,13 @@
 #
 # Convert a factor to a matrix of dummy variables, etc.
 #
-#  $Revision: 1.3 $  $Date: 2011/10/11 02:36:15 $
+#  $Revision: 1.5 $  $Date: 2016/02/11 10:17:12 $
 #
 
 dummify <- function(x) {
   if(is.matrix(x) || is.data.frame(x)) {
     x <- as.data.frame(x)
-    y <- do.call("data.frame", lapply(x, dummify))
+    y <- do.call(data.frame, lapply(x, dummify))
     return(as.matrix(y))
   }
   # x is 1-dimensional
