@@ -1,7 +1,7 @@
 #
 # mppm.R
 #
-#  $Revision: 1.78 $   $Date: 2016/04/24 07:16:59 $
+#  $Revision: 1.80 $   $Date: 2016/07/06 07:06:03 $
 #
 
 mppm <- local({
@@ -54,7 +54,8 @@ mppm <- local({
     ##------  Trend Formula ------------------
     
     ## check all variables in trend formula are recognised
-    checkvars(formula, data.sumry$col.names, extra=c("x","y","id"), bname="data")
+    checkvars(formula, data.sumry$col.names,
+              extra=c("x","y","id","marks"), bname="data")
     ## check formula has LHS and RHS. Extract them
     if(length(formula) < 3)
       stop(paste("Argument", sQuote("formula"),
@@ -641,7 +642,7 @@ simulate.mppm <- function(object, nsim=1, ..., verbose=TRUE) {
                                                   nsim=nsim, drop=FALSE),
                                              list(...),
                                              list(progress=FALSE)))
-    if(verbose) progressreport(irow, nr)
+    if(verbose) state <- progressreport(irow, nr, state=state)
   }
   sim1list <- lapply(sims, "[[", i=1)
   h <- hyperframe("Sim1"=sim1list)
