@@ -6,9 +6,10 @@
 #include <stdlib.h>
 #include <time.h>
 #include <R.h>
-#include <Rmath.h>
 #include <Rdefines.h>
+#include <Rmath.h>
 #include <R_ext/Utils.h>
+#include <R_ext/Constants.h>
 
 // #include <stdio.h>
 // FILE *out;
@@ -845,3 +846,4 @@ void Sampler::Sim(Point2Pattern *p2p, long int *ST, long int *ET) {
 #include "PerfectHardcore.h"
 #include "PerfectDiggleGratton.h"
 #include "PerfectDGS.h"
+#include "PerfectPenttinen.h"
